@@ -1,3 +1,12 @@
+use std::{
+    fs,
+    sync::atomic::{AtomicUsize, Ordering},
+    thread, time,
+};
+
+use tree_sitter::{IncludedRangesError, InputEdit, LogType, Parser, Point, Range};
+use tree_sitter_proc_macro::retry;
+
 use super::helpers::{
     allocations,
     edits::{invert_edit, ReadRecorder},
@@ -8,13 +17,6 @@ use crate::{
     parse::{perform_edit, Edit},
     tests::helpers::fixtures::fixtures_dir,
 };
-use std::{
-    fs,
-    sync::atomic::{AtomicUsize, Ordering},
-    thread, time,
-};
-use tree_sitter::{IncludedRangesError, InputEdit, LogType, Parser, Point, Range};
-use tree_sitter_proc_macro::retry;
 
 #[test]
 fn test_parsing_simple_string() {
@@ -97,7 +99,7 @@ fn test_parsing_with_debug_graph_enabled() {
     parser.print_dot_graphs(&debug_graph_file);
     parser.parse("const zero = 0", None).unwrap();
 
-    debug_graph_file.seek(std::io::SeekFrom::Start(0)).unwrap();
+    debug_graph_file.rewind().unwrap();
     let log_reader = BufReader::new(debug_graph_file)
         .lines()
         .map(|l| l.expect("Failed to read line from graph log"));
@@ -817,9 +819,19 @@ fn test_parsing_with_one_included_range() {
     let script_content_node = html_tree.root_node().child(1).unwrap().child(1).unwrap();
     assert_eq!(script_content_node.kind(), "raw_text");
 
+    assert_eq!(
+        parser.included_ranges(),
+        &[Range {
+            start_byte: 0,
+            end_byte: u32::MAX as usize,
+            start_point: Point::new(0, 0),
+            end_point: Point::new(u32::MAX as usize, u32::MAX as usize),
+        }]
+    );
     parser
         .set_included_ranges(&[script_content_node.range()])
         .unwrap();
+    assert_eq!(parser.included_ranges(), &[script_content_node.range()]);
     parser.set_language(&get_language("javascript")).unwrap();
     let js_tree = parser.parse(source_code, None).unwrap();
 
