@@ -1,5 +1,9 @@
 var TreeSitter = function() {
   var initPromise;
+  var document = typeof window == 'object'
+    ? {currentScript: window.document.currentScript}
+    : null;
+
   class Parser {
     constructor() {
       this.initialize();
@@ -11,5 +15,5 @@ var TreeSitter = function() {
 
     static init(moduleOptions) {
       if (initPromise) return initPromise;
-      Module = Object.assign({ }, Module, moduleOptions);
+      Module = Object.assign({}, Module, moduleOptions);
       return initPromise = new Promise((resolveInitPromise) => {
